package miner

import (
	"context"

	"github.com/filecoin-project/go-address"
	"github.com/filecoin-project/lotus/api"
	"github.com/filecoin-project/lotus/chain/gen"
	"github.com/filecoin-project/specs-actors/actors/abi"
	lru "github.com/hashicorp/golang-lru"
)

type MineReq struct {
	InjectNulls abi.ChainEpoch
	Done        func(bool, error)
}

func NewTestMiner(nextCh <-chan MineReq, addr address.Address) func(api.FullNode, gen.WinningPoStProver) *Miner {
	return func(api api.FullNode, epp gen.WinningPoStProver) *Miner {
		arc, err := lru.NewARC(10000)
		if err != nil {
			panic(err)
		}

		m := &Miner{
			api:               api,
			waitFunc:          chanWaiter(nextCh),
			epp:               epp,
			minedBlockHeights: arc,
			address:           addr,
		}

		if err := m.Start(context.TODO()); err != nil {
			panic(err)
		}
		return m
	}
}

func chanWaiter(next <-chan MineReq) func(ctx context.Context, _ uint64) (func(bool, error), abi.ChainEpoch, error) {
	return func(ctx context.Context, _ uint64) (func(bool, error), abi.ChainEpoch, error) {
		select {
		case <-ctx.Done():
			return nil, 0, ctx.Err()
		case req := <-next:
			return req.Done, req.InjectNulls, nil
		}
	}
}
